import { BorderSize, TopicType } from "@/enum";
import { MindMapProjectInfo, TopicItem } from "@/types";
import { Graph, Cell, Node } from "@antv/x6";
import TopicComponent from "@/components/mindMap/Topic";
import Hierarchy from "@antv/hierarchy";
import { topicData } from "@/config/data";
import { uuid } from "@/utils";
import { SetState } from "ahooks/lib/useSetState";

interface HierarchyResult {
  id: string;
  x: number;
  y: number;
  data: TopicItem;
  children?: HierarchyResult[];
}
/**
 * 渲染思维导图项目
 * @param graph 
 */
export const renderMindMap = (
  graph: Graph,
  setMindProjectInfo: () => void
) => {
  const projectInfo = getMindMapProjectByLocal();
  if(!projectInfo) return;
  const { topics, pageSetting } = projectInfo;
  const cells: Cell[] = [];
  topics.forEach((topic) => {
    // 遍历出层次结构
    const result: HierarchyResult = Hierarchy.mindmap(topic, {
      direction: "H",
      getHeight(d: TopicItem) {
        return d.height;
      },
      getWidth(d: TopicItem) {
        return d.width;
      },
      getHGap(d: TopicItem) {
        if (d.type === TopicType.main) return pageSetting.branchX;
        if (d.type === TopicType.branch) return pageSetting.subTopicX;
        if (d.type === TopicType.sub) return pageSetting.subTopicX;
        return 40;
      },
      getVGap(d: TopicItem) {
        if (d.type === TopicType.main) return pageSetting.subTopicY;
        if (d.type === TopicType.branch) return pageSetting.subTopicY;
        if (d.type === TopicType.sub) return pageSetting.subTopicY;
        return 20;
      },
      getSide: () => {
        return "right";
      },
    });
    const traverse = (
      hierarchyItem: HierarchyResult
    ) => {
      if (hierarchyItem) {
        const { data, children, x, y } = hierarchyItem;
        const id = data?.id || uuid();
        // 创建主题
        cells.push(
          graph.createNode({
            ...TopicComponent,
            width: data.width,
            height: data.height,
            data: {
              ...data,
              // 节点内部执行数据更新方法
              setMindProjectInfo,
            },
            id,
            x,
            y,
          })
        );

        if (children) {
          children.forEach((item: HierarchyResult) => {
            cells.push(
              // 创建连线
              graph.createEdge({
                shape: "mindmap-edge",
                source: {
                  cell: id,
                  anchor: {
                    name: "center",
                    args: {
                      dx: "25%",
                    },
                  },
                },
                target: {
                  cell: item.id,
                  anchor: {
                    name: "left",
                  },
                },
              })
            );
            // 递归遍历
            traverse(item);
          });
        }
      }
    };
    
    traverse(result);
  });

  cells.forEach((cell) => {
    // 存在更新位置，否则添加
    if (graph.hasCell(cell.id) && cell.isNode()) {
      const oldCell = graph.getCellById(cell.id);
      oldCell.isNode() && oldCell.position(cell.position().x, cell.position().y);
    } else {
      graph.addCell(cell);
    }
  });
  // graph.centerContent();
};

/**
 * 添加分支主题
 */
export const addTopic = (
  node: Node,
  type: TopicType,
  otherData: Record<string, any> = {}
) => {
  const projectInfo = getMindMapProjectByLocal();
  if (!projectInfo) return;

  const topic = buildTopic(type, otherData);
  const parentData = node.getData();
  const traverse = (topics: TopicItem[]) => {
    topics.forEach((item) => {
      if (item.id === parentData.id) {
        if (item.children) {
          item.children?.push(topic);
        } else {
          item.children = [topic];
        }
      }
      if (item.children) {
        traverse(item.children);
      }
    });
  };

  traverse(projectInfo?.topics || []);

  parentData?.setMindProjectInfo(projectInfo);
};

const topicMap = {
  [TopicType.main]: "中心主题",
  [TopicType.branch]: "分支主题",
  [TopicType.sub]: "子主题"
};

/**
 * 构建一个主题数据
 * @param type 主题类型
 * @param options 配置项
 * @returns
 */
export const buildTopic = (
  type: TopicType,
  options: Record<string, any> = {}
) => {
  const projectInfo = getMindMapProjectByLocal();

  const theme = projectInfo?.theme || {};
  return {
    ...topicData,
    id: uuid(),
    type,
    label: topicMap[type] || "自由主题",
    width: theme[type]?.width || 206,
    height: theme[type]?.height || 70,
    fill: {
      ...topicData.fill,
      ...theme[type]?.fill,
    },
    text: {
      ...topicData.text,
      ...theme[type]?.text,
    },
    stroke: {
      ...topicData.stroke,
      ...theme[type]?.stroke,
    },
    ...options,
  };
};

/**
 * 从本地获取项目信息
 * @returns
 */
export const getMindMapProjectByLocal = (): MindMapProjectInfo | null => {
  return JSON.parse(localStorage.getItem("minMapProjectInfo") || "null");
};
